% Plot figures for 2013 ISPLC paper entitled "Cyclic Spectral Analysis of
% Power Line Noise in the 3-200 kHz Band"
% K. Nieman, 2/11/13

close all;

% File paths
fname = {'../data/2012-10-23_noise_analysis/ens_414d/rx_line_samples_02.bin',...
         '../data/2012-10-23_noise_analysis/ens_607/rx_line_samples_01.bin',...
         '../data/2012-10-23_noise_analysis/chilling_station/rx_line_samples_01.bin',...
         '../data/2012-10-23_noise_analysis/apartment/rx_line_samples_01.bin'};
figpath = './figs';

% Disable warning shown by Antoni's cyclic analysis functions
warning('off','MATLAB:warn_r14_stucture_assignment');

% Hardware constants
counts2volts = 1.8/25e3;
fs = 400e3;

% FFT window size, overlap, and length
Nw = 256;
Nv = fix(2*Nw/3);
Nfft = 2*Nw;
    
% Perform analysis for all samples in 'fname'
for filenum = 1:length(fname)
    % Convert x from counts to volts; equalize samples
    x = counts2volts*readSamplesFile(fname{:,filenum},'complex');
    x = equalizeLineSamples(x,'eq_coefs.bin');
    t = (0:length(x) - 1).'/fs;
    
    % Compute PSD of samples
    [S,F,T,P] = spectrogram(x,hamming(Nw),Nv,Nfft,fs);
    P(F >= fs/2,:) = [];
    F(F >= fs/2) = [];
    
    % Open Figure
    figure(filenum)
    
    % Plot PSD/time-domain trace in Fig #(a)
    scrsz = get(0,'ScreenSize');
    set(gcf,'Position',[(5 + 405*(filenum-1)) (scrsz(4) - 1045) 400 968])
    
    % Fig #(a), axis 1 = PSD
    h(1) = axes('Position',[0.13 0.745 0.85 0.19]);
    imagesc(T*1e3,F/1e3,10*log10(abs(P)));
    annotation( 'textbox',[.15 .895, .07, .03],...
                'String','(a)',...
                'BackgroundColor',[1 1 1]);
    set(gca,'Ydir','normal');
    apos = get(gca,'Position');
    xlabel('{\itt} (ms)');
    ylabel('{\itf} (kHz)');
    cb = colorbar(  'peer',gca,...
                    'location','NorthOutside',...
                    'XAxisLocation','bottom',...
                    'Position',[apos(1) 0.98 apos(3) 0.01]);
    set(get(cb,'xlabel'),...
        'String','spectral density (dBV/Hz^{0.5})');
    set(gca,'Position',[0.13 0.745 0.85 0.19])

    % Fig #(a), axis 2 = time-domain trace
    h(2) = axes('Position',[0.13 0.675 0.85 0.069]);
    n = (t < T(1)) | (t > T(end));
    t(n) = [];
    t(1) = t(1) - (T(2)-T(1))/2;
    t(end) = t(end) + (T(2)-T(1))/2;
    t = t*1e3;
    plot(t,real(x(~n)));
    axis([min(t) max(t) -2 2]);
    xlabel('{\itt} (ms)');
    ylabel('V');
    set(gca,'YTick',[-1 0 1])
    
    % Parameters for cyclic spectral analysis functions
    L = length(x);
    da = 1/L;       % cyclic frequency resolution
    a1 = 1;         % first cyclic frequency bin to scan
    a2 = 150;       % last cyclic frequency bin to scan

    % Computer cyclic spectral density and cyclic spectral coherence
    C = zeros(Nfft,a2-a1+1);
    S = zeros(Nfft,a2-a1+1);
    for k = a1:a2;
        Coh = SCoh_W(x,x,k/L,Nfft,Nv,Nw,'sym',.01);
        Spec = CPS_W(x,x,k/L,Nfft,Nv,Nw,'sym',0.01);
        C(:,k-a1+1) = Coh.C;
        S(:,k-a1+1) = Coh.S;
        waitbar((k-a1+1)/(a2-a1+1))
    end

    % Upsample for smoother meshplot
    K_alpha = 8;
    S = (1/fs)*ifft([K_alpha*fft(S,[],2),zeros(size(S,1),size(S,2)*(K_alpha - 1))],[],2);
    C = ifft([K_alpha*fft(C,[],2),zeros(size(C,1),size(C,2)*(K_alpha - 1))],[],2);
    az = 120;       % azimuth angle (from x-z plane) to camera in degrees
    el = 60;        % elevation angle (from x-y plane) to camera in degrees
    alpha = (a1:1/K_alpha:a2 + (K_alpha - 1)/K_alpha)*da;

    % Fig #(b), cyclic spectral density
    axes('Position',[0.13 0.34 0.85 0.22]);
    [X,Y] = meshgrid(fs*alpha/1e3,fs*Coh.f(1:Nfft/2)/1e3);
    mesh(   X,Y,10*log10(abs(S(1:Nfft/2,:))),...
            'EdgeColor','interp',...
            'FaceColor','interp');
    axis tight;
    h = xlabel('\alpha (kHz)');
    set(h,'Rotation',sin(pi*el/180)*(180-az) - 8)
    h = ylabel('{\itf} (kHz)');
    set(h,'Rotation',cos(pi*el/180)*(90-az) + 6)
    set(gca,'View',[az el])
    xlabh = get(gca,'XLabel');
    xlabpos = get(xlabh,'Position');
    set(xlabh,'Position',xlabpos.*[0.98 0.97 1]);
    ylabh = get(gca,'YLabel');
    ylabpos = get(ylabh,'Position');
    set(ylabh,'Position',ylabpos.*[0.98 0.92 1]);
    apos = get(gca,'Position');
    cb = colorbar(  'peer',gca,...
                    'location','NorthOutside',...
                    'XAxisLocation','bottom',...
                    'Position',[apos(1) 0.604 apos(3) 0.01]);
    set(get(cb,'xlabel'),...
        'String','cyclic spectral density (dBV/Hz^{0.5})');
    set(gca,'Position',[0.13 0.34 0.85 0.22]);
    annotation( 'textbox', [.15 .52, .07, .03], ...
                'String', '(b)', ...
                'BackgroundColor',[1 1 1]);
    
    % Fig #(c), cyclic spectral coherence        
    axes('Position',[0.13 0.02 0.85 0.22]);
    mesh(   X,Y,abs(C(1:Nfft/2,:)),...
            'EdgeColor','interp',...
            'FaceColor','interp');
    axis tight;
    caxis([0.15 0.75])
    h = xlabel('\alpha (kHz)');
    set(h,'Rotation',sin(pi*el/180)*(180-az) - 8)
    h = ylabel('{\itf} (kHz)');
    set(h,'Rotation',cos(pi*el/180)*(90-az) + 6)
    set(gca,'View',[az el])
    xlabh = get(gca,'XLabel');
    xlabpos = get(xlabh,'Position');
    set(xlabh,'Position',xlabpos.*[0.98 0.97 1]);
    ylabh = get(gca,'YLabel');
    ylabpos = get(ylabh,'Position');
    set(ylabh,'Position',ylabpos.*[0.98 0.92 1]);
    apos = get(gca,'Position');
    cb = colorbar(  'peer',gca,...
                    'location','NorthOutside',...
                    'XAxisLocation','bottom',...
                    'Position',[apos(1) 0.284 apos(3) 0.01]);
    set(get(cb,'xlabel'),'String', 'cyclic spectral coherence');
    set(gca,'Position',[0.13 0.02 0.85 0.22])
    annotation( 'textbox',[.15 .205, .07, .03],...
                'String', '(c)',...
                'BackgroundColor',[1 1 1]);

    % Format Fig # for printing to pdf
    set(gcf,'Units','centimeters');	
    sz = get(gcf, 'Position');
    sz(1:2) = 0;
    set(gcf,'PaperUnits','centimeters',...
            'PaperSize',sz(3:4),...
            'PaperPosition', sz);

    % Print Fig # to pdf
    figname = sprintf('fig%02d',filenum + 1);
    print('-dpdf','-r600',fullfile(figpath,figname));
end

% Fig (filenum + 2)(a), in-band noise of last file
figure(filenum + 1);
x = counts2volts*readSamplesFile(fname{:,filenum},'complex');
x = equalizeLineSamples(x,'eq_coefs.bin');
set(gcf,'Position',[5 (scrsz(4) - 300) 400 203])

% Design bandpass filter for G3 CENELEC-A operation
M = 10;
t = (0:length(x) - 1).'/fs;
[b,a] = ellip(M,0.5,80,53/400);
b = b.*exp(j*2*pi*(0:M).*62.5e3/fs);
a = a.*exp(j*2*pi*(0:M).*62.5e3/fs);

% G3 CENELEC-A parameters
Nfft = 256;
Nw = 256;
Nv = -22;   % negative overlap effectively removes CP
startSym = 1;
g3tones = 23:58;

% Determine in-band SNR per subcarrier
axes('Position',[0.13 0.18 0.68 0.8]);
[S,F,T,P] = spectrogram(complex(real(x)),ones(Nw,1),Nv,Nfft,fs);
PdB = 10*log10(P(g3tones,startSym:end));
avgSNR = 10*log10(mean(mean(P(g3tones,startSym:end))));
imagesc(1:size(PdB,2),g3tones,PdB - avgSNR);
set(gca,'YDir','normal')
xlabel('OFDM Symbol');
ylabel('subcarrier');
apos = get(gca,'Position');
cb = colorbar(  'peer',gca,...
                'location','EastOutside',...
                'Position',[0.85 apos(2) 0.02 apos(4)]);
set(get(cb,'ylabel'),'String','noise power vs avg (dB)');
annotation( 'textbox',[.143 .83, .07, .13],...
            'String','(a)','BackgroundColor',[1 1 1]);

% Format Fig (filenum + 2)(a) for printing to pdf
set(gcf,'Units','centimeters');	
sz = get(gcf, 'Position');
sz(1:2) = 0;
set(gcf,'PaperUnits','centimeters',...
        'PaperSize',sz(3:4),...
        'PaperPosition', sz);
    
% Print Fig (filenum + 2)(a) to pdf
figname = sprintf('fig%02da',filenum + 2);
print('-dpdf','-r600',fullfile(figpath,figname));

% Cyclic modulation parameters
op = [0;6;7;10];    % 10^-2 BER operating point for ROBO6,DBPSK,DQPSK,D8PSK
bits = [1/4;1;2;3]; % bits-per-symbol
M = length(op) + 1; 
dM = 1/(M-1);

% Fig (filenum + 2)(b), optimal modulation
figure(filenum + 2);
set(gcf,'Position',[5 (scrsz(4) - 580) 400 203])
h = axes('Position',[0.13 0.18 0.68 0.8]);
hdrm = -7 + avgSNR - PdB;
mod = zeros(size(hdrm));
nn = 0;
adaptBits = 0;
staticBits = zeros(length(op),1);

% Loop through different modulation types, find max supported rate
for k = 1:length(op)
    if k == length(op)
        n = hdrm >= op(k);
    else
        n = (hdrm >= op(k)) & (hdrm < op(k + 1));
    end
    adaptBits = adaptBits + sum(sum(n))*bits(k);
    mod(n) = dM*k;
end

% Compute rate supported by static modulation with tone mask
pthresh = 0.5;
for k = 1:length(op)
    n = (hdrm >= op(k));
    n(sum(n,2) < pthresh*size(hdrm,2),:) = 0;
    staticBits(k) = sum(sum(n))*bits(k);
end

% Compute rate supported by cyclic scheme
cycAdaptBits = 0;
mask = repmat(mod(:,1:12)/dM,1,10);
mask = mask(:,(1:size(hdrm,2)));
for k = 1:length(op)
    if k == length(op)
        n = hdrm(mask == k) >= op(k);
    else
        n = (hdrm(mask == k) >= op(k)) & (hdrm(mask == k) < op(k + 1));
    end
    cycAdaptBits = cycAdaptBits + sum(sum(n))*bits(k);
end

% Create visual map of optimal modulation types
imagesc(1:size(PdB,2),g3tones,mod);
set(gca,'YDir','normal')
xlabel('OFDM Symbol');
ylabel('subcarrier');
apos = get(gca,'Position');
cb = colorbar('peer',gca,'location','EastOutside','Position',[0.85 apos(2) 0.02 apos(4)]);
colormap([0 0 0;0 0 1;0 1 1;1 0 1;0 1 0]);
set(cb, 'YTickLabel',{'NONE','ROBO','DBPSK','DQPSK','D8PSK'},...
        'YTick',(0.1:0.2:1).');
annotation( 'textbox', [.143 .83, .07, .13],...
            'String', '(b)',...
            'BackgroundColor',[1 1 1]);

% Format Fig (filenum + 2)(b) for printing to pdf
set(gcf,'Units','centimeters');	
sz = get(gcf, 'Position');
sz(1:2) = 0;
set(gcf,'PaperUnits','centimeters',...
        'PaperSize',sz(3:4),...
        'PaperPosition', sz);
    
% Print Fig (filenum + 2)(b) to pdf
figname = sprintf('fig%02db',filenum + 2);
print('-dpdf','-r600',fullfile(figpath,figname));

% Fig (filenum + 2)(c), G3 cyclic modulation scheme
figure(filenum + 3);
set(gcf,'Position',[5 (scrsz(4) - 860) 400 203])
h = axes('Position',[0.13 0.18 0.85 0.79]);
theoretical_max = 1e-3*[(1/4);1;2;3;3]*36/((256+22)/400e3);
bar(theoretical_max,'r');hold on
set(gca,'XTickLabel',{'ROBO','DBPSK','DQPSK','D8PSK','cyclic'},'YScale','log');
set(get(gca,'Children'),'LineWidth',1,...
                        'FaceColor',[245 245 245]/255,...
                        'LineStyle','-',...
                        'BarWidth',0.6)
bar(1e-3*[staticBits;cycAdaptBits]/(length(T)*(256 + 22)/400e3),'r');
set(get(gca,'Children'),'BarWidth',0.6)
plot([-1;6],repmat(1e-3*adaptBits/(length(T)*(256 + 22)/400e3),2,1),...
            'b--','LineWidth',2);
hold off;
ylabel('uncoded throughput (kbps)');
legend('max supported','achieved','Location','NorthWest')
axis([-.35 5.65 0 500])
xlabel('modulation')
annotation( 'textbox',[.143 .205, .07, .13],...
            'String','(c)',...
            'BackgroundColor',[1 1 1]);
y = 1e-3*[staticBits;cycAdaptBits]/(length(T)*(256 + 22)/400e3);
for i=1:length(y),
  TH(i) = text(i,y(i),num2str(floor(y(i))));
end
set(TH,'Horizontalalignment','center','FontSize',6,...
'verticalalignment','bottom') ;
y = 1e-3*adaptBits/(length(T)*(256 + 22)/400e3);
TH = text(-.35,y,sprintf('  optimal adaptive = %d',floor(y)));
set(TH, 'Horizontalalignment','left',...
        'FontSize',8,...
        'verticalalignment','top');

% Format Fig (filenum + 2)(c) for printing to pdf
set(gcf,'Units','centimeters');	
sz = get(gcf, 'Position');
sz(1:2) = 0;
set(gcf,'PaperUnits','centimeters',...
        'PaperSize',sz(3:4),...
        'PaperPosition', sz);
    
% Print Fig (filenum + 2)(c) to pdf
figname = sprintf('fig%02dc',filenum + 2);
print('-dpdf','-r600',fullfile(figpath,figname));







